@@ -6,10 +6,17 @@ import pickle
 import base64
 from multiprocessing import Process, JoinableQueue
 import subprocess
+from collections import Counter
+import configparser
+config = configparser.ConfigParser()
+import csv
 
-def write2path(lines,path,encoding='utf-8',method='lines',append='w'):
+def write2path(lines,path='',encoding='utf-8',method='lines',append='w',wrap=False):
+    if not path: path=getTime()+'.txt'
     with open(path,append,encoding=encoding) as fw:
         if method=='lines':
+            lines = [str(f) for f in lines]
+            if wrap: lines = [f+'\n' for f in lines]
             fw.writelines(lines)
         else:
             fw.write(lines)
@@ -21,10 +28,12 @@ def append2path(line,path,encoding='utf-8',method='line',append='a'):
         else:
             fw.writelines(line)
 
-def read_path(path,method='readlines',encoding='utf-8',errors='strict'):
+def read_path(path,method='lines',encoding='utf-8',errors='strict',strip=False):
     with open(path,'r',encoding=encoding,errors=errors) as fr:
-        if method == "readlines":
+        if method == "lines":
             all_lines = fr.readlines()
+            if strip:
+                all_lines = [l.strip() for l in all_lines]
         else:
             all_lines = fr.read()
         return all_lines
@@ -36,10 +45,10 @@ def print_list(array,index=False):
     else:
         for val in array:
             print(val)
+    print(f"length:{len(array)}")
 
 def get_top_n_files(folder_path,n):
     if not os.path.exists(folder_path):
-        print('源文件夹不存在.')
         return []
     if folder_path[-1] != '/':
         folder_path=folder_path+'/'
@@ -62,16 +71,18 @@ def read_pkl(path):
         obj = pickle.load(fr)
         return obj
 
-def getTime(timestamp=-1):
-    if timestamp==-1:
-        return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
+def getTime(timestamp=''):
+    if timestamp:
+        return time.strftime("%y-%m-%d %H:%M:%S", time.localtime(timestamp))
     else:
-        return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(timestamp))
+        return time.strftime("%y-%m-%d %H:%M:%S", time.localtime())
+
+def getTimestamp():
+    return time.time()
 
 def getTimeSpan(begin_time, end_time, format='minute'):
-    # 获取2个字串串格式的时间差
-    begin_time = time.strptime(begin_time, "%Y-%m-%d %H:%M:%S")
-    end_time = time.strptime(end_time, "%Y-%m-%d %H:%M:%S")
+    begin_time = time.strptime(begin_time, "%y-%m-%d %H:%M:%S")
+    end_time = time.strptime(end_time, "%y-%m-%d %H:%M:%S")
 
     begin_timeStamp = int(time.mktime(begin_time))
     end_timeStamp = int(time.mktime(end_time))
@@ -99,17 +110,33 @@ def get_file_md5(file_path):
         md5 = str(hash_code).lower()
     return md5
 
+def get_str_md5(parmStr):
+    if isinstance(parmStr, str):
+        parmStr = parmStr.encode("utf-8")
+    m = hashlib.md5()
+    m.update(parmStr)
+    return m.hexdigest()
+
 def getfilesize(filePath):
     fsize = os.path.getsize(filePath)
     fsize = fsize / (1000 * 1000)
     return round(fsize, 2)
 
-def listDir(rootDir):
+def listDir(rootDir,only_file=False,only_folder=False):
     list_filepath = []
     for filename in os.listdir(rootDir):
         pathname = os.path.join(rootDir, filename)
-        if os.path.isfile(pathname):
-            list_filepath.append(pathname)
+        if filename == '.DS_Store':
+            continue
+        if only_file:
+            if os.path.isfile(pathname):
+                list_filepath.append(pathname)
+            continue
+        if only_folder:
+            if os.path.isdir(pathname):
+                list_filepath.append(pathname)
+            continue
+        list_filepath.append(pathname)
     return list_filepath
 
 def makedir(dir_path,delete_exists=False):
@@ -127,6 +154,9 @@ def touchfile(path):
         f = open(path,'w')
         f.close()
 
+def counter(lst):
+    return Counter(lst)
+
 def copyfile(origin_path, target_path):
     if os.path.isfile(origin_path):
         shutil.copy(origin_path, target_path)
@@ -178,10 +208,19 @@ def run_cmd(cmd,with_output=False):
     else:
         return subprocess.call(cmd, shell=True)
 
-def run_multi_task(all_items,done_path,user_func,cpu_num=os.cpu_count(),append_done=True):
+def func_time(func):
+    begin_time= time.time()
+    func()
+    end_time= time.time()
+    print(f'{func.__name__} 耗时:{int(end_time-begin_time)}')
+
+def run_multi_task(all_items,user_func,done_path='',cpu_num=os.cpu_count(),print_log=False):
     begin_time = time.time()
-    touchfile(done_path)
-    done_items = [f.strip() for f in read_path(done_path)]
+    if done_path:
+        touchfile(done_path)
+        done_items = [f.strip() for f in read_path(done_path)]
+    else:
+        done_items = []
     undone_items = list(set(all_items) - set(done_items))
     print(f'检测到全部:{len(all_items)}个')
     print(f'检测到已完成的:{len(done_items)}个')
@@ -196,14 +235,79 @@ def run_multi_task(all_items,done_path,user_func,cpu_num=os.cpu_count(),append_d
         while True:
             item = q.get()
             user_func(item)
-            if append_done:
-                append2path(item+'\n',done_path)
+            if done_path:
+                append2path(f'{item}',done_path)
+
             q.task_done()
+            if print_log:
+                print(f'当前:{item}')
 
     for i in range(cpu_num):
         p = Process(target=func_task, args=(q,))
         p.daemon = True
         p.start()
     q.join()
-    print(f'全部已完成，用时:{int(time.time() - begin_time)}')
+    print(f'全部已完成，用时:{float(time.time() - begin_time)}')
 
+def read_config(config_path,config_name,section='default',type='str'):
+    config = configparser.ConfigParser()
+    config.read(config_path)
+    if section in config.sections():
+        if config_name in config.options(section):
+            if type=='str':
+                return config.get(section, config_name)
+            elif type=='float':
+                return config.getfloat(section, config_name)
+            elif type=='int':
+                return config.getint(section, config_name)
+            elif type=='bool':
+                return config.getboolean(section, config_name)
+    return ''
+
+def write_config(config_path,config_name,config_value,section='default'):
+    config = configparser.ConfigParser()
+    config.read(config_path)
+    if not config.has_section(section): config.add_section(section)
+    config.set(section, config_name, config_value)
+
+    with open(config_path, 'w') as configfile:
+        config.write(configfile)
+
+def read_csv(csv_path,method="list",encoding='utf-8',header=False):
+    '''
+    :param method: list or dict
+    :param return: [[1, 'chen', 'male']]  or [[('age', '1'), ('name', 'chen'), ('sex', 'male')]]
+    '''
+    csv_cnts=[]
+    headers=[]
+    with open(csv_path, encoding=encoding) as f:
+        if method=="list":
+            reader = csv.reader(f)
+            headers.extend(next(reader))
+            for row in reader:
+                csv_cnts.append(row)
+        else:
+            reader = csv.DictReader(f)
+            headers.extend(list(next(reader).keys()))
+            for row in reader:
+                csv_cnts.append(row)
+    if header:
+        return headers,csv_cnts
+    else:
+        return csv_cnts
+
+def write_csv(csv_path,header=[],rows=[],method="list",encoding='utf-8',append='w'):
+    '''
+    :param header: ['name', 'age', 'sex']
+    :param rows: [{'age': 1, 'name': 'chen', 'sex': 'male'}] or [[1, 'chen', 'male']]
+    :param method: list or dict
+    '''
+    with open(csv_path, append, encoding=encoding, newline='') as f:
+        if method=="list":
+            writer = csv.writer(f)
+            writer.writerow(header)
+            writer.writerows(rows)
+        else:
+            writer = csv.DictWriter(f, header)
+            writer.writeheader()
+            writer.writerows(rows)
